@@ -102,12 +102,13 @@ cddl_types_from_ast! {
 }
 
 #[cfg(feature = "ast-comments")]
+#[cfg_attr(target_arch = "wasm32", derive(Serialize))]
 #[derive(Default, Debug, PartialEq, Eq, Clone)]
 #[doc(hidden)]
 pub struct Comments<'a>(pub Vec<&'a str>);
 
 #[cfg(feature = "ast-comments")]
-impl<'a> Comments<'a> {
+impl Comments<'_> {
   fn any_non_newline(&self) -> bool {
     self.0.iter().any(|c| *c != "\n")
   }
@@ -118,7 +119,7 @@ impl<'a> Comments<'a> {
 }
 
 #[cfg(feature = "ast-comments")]
-impl<'a> fmt::Display for Comments<'a> {
+impl fmt::Display for Comments<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     if self.all_newline() {
       return write!(f, "");
@@ -151,12 +152,11 @@ pub struct CDDL<'a> {
   pub rules: Vec<Rule<'a>>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for CDDL<'a> {
+impl fmt::Display for CDDL<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     #[cfg(target_arch = "wasm32")]
     console_error_panic_hook::set_once();
@@ -228,15 +228,15 @@ pub struct Identifier<'a> {
   pub span: Span,
 }
 
-impl<'a> PartialEq for Identifier<'a> {
+impl PartialEq for Identifier<'_> {
   fn eq(&self, other: &Self) -> bool {
     self.to_string() == other.to_string()
   }
 }
 
-impl<'a> Eq for Identifier<'a> {}
+impl Eq for Identifier<'_> {}
 
-impl<'a> fmt::Display for Identifier<'a> {
+impl fmt::Display for Identifier<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     if let Some(sp) = &self.socket {
       return write!(f, "{}{}", sp, self.ident);
@@ -246,7 +246,7 @@ impl<'a> fmt::Display for Identifier<'a> {
   }
 }
 
-impl<'a> From<&'static str> for Identifier<'a> {
+impl From<&'static str> for Identifier<'_> {
   fn from(ident: &'static str) -> Self {
     let mut socket = ident.chars().take(2);
 
@@ -283,11 +283,7 @@ impl<'a> From<&'static str> for Identifier<'a> {
 
 impl<'a> From<Token<'a>> for Identifier<'a> {
   fn from(token: Token) -> Self {
-    let token = if let Some(token) = token.in_standard_prelude() {
-      token
-    } else {
-      ""
-    };
+    let token = token.in_standard_prelude().unwrap_or_default();
 
     Identifier::from(token)
   }
@@ -312,7 +308,6 @@ pub enum Rule<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_rule: Option<Comments<'a>>,
   },
@@ -325,13 +320,12 @@ pub enum Rule<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_rule: Option<Comments<'a>>,
   },
 }
 
-impl<'a> Rule<'a> {
+impl Rule<'_> {
   /// Return `Span` for `Rule`
   #[cfg(feature = "ast-span")]
   pub fn span(&self) -> Span {
@@ -384,7 +378,7 @@ impl<'a> Rule<'a> {
   }
 }
 
-impl<'a> fmt::Display for Rule<'a> {
+impl fmt::Display for Rule<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     match self {
       Rule::Type {
@@ -437,7 +431,7 @@ impl<'a> fmt::Display for Rule<'a> {
   }
 }
 
-impl<'a> Rule<'a> {
+impl Rule<'_> {
   /// Returns the name id of a rule
   pub fn name(&self) -> String {
     match self {
@@ -475,16 +469,14 @@ pub struct TypeRule<'a> {
   pub value: Type<'a>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_assignt: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_assignt: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for TypeRule<'a> {
+impl fmt::Display for TypeRule<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut tr_output = self.name.to_string();
 
@@ -533,16 +525,14 @@ pub struct GroupRule<'a> {
   pub entry: GroupEntry<'a>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_assigng: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_assigng: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for GroupRule<'a> {
+impl fmt::Display for GroupRule<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut gr_output = self.name.to_string();
 
@@ -595,16 +585,14 @@ pub struct GenericParam<'a> {
   pub param: Identifier<'a>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_ident: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_ident: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for GenericParams<'a> {
+impl fmt::Display for GenericParams<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut gp = String::from("<");
     for (idx, parm) in self.params.iter().enumerate() {
@@ -654,16 +642,14 @@ pub struct GenericArg<'a> {
   pub arg: Box<Type1<'a>>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_type: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_type: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for GenericArgs<'a> {
+impl fmt::Display for GenericArgs<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut ga = String::from("<");
     for (idx, arg) in self.args.iter().enumerate() {
@@ -763,16 +749,14 @@ pub struct TypeChoice<'a> {
   /// Type choice
   pub type1: Type1<'a>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_type: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_type: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for Type<'a> {
+impl fmt::Display for Type<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut type_str = String::new();
 
@@ -873,7 +857,6 @@ pub struct Type1<'a> {
   pub span: Span,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_type: Option<Comments<'a>>,
 }
@@ -941,16 +924,14 @@ pub struct Operator<'a> {
   pub type2: Type2<'a>,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_operator: Option<Comments<'a>>,
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_after_operator: Option<Comments<'a>>,
 }
 
-impl<'a> fmt::Display for Type1<'a> {
+impl fmt::Display for Type1<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut t1_str = String::new();
 
@@ -1143,11 +1124,9 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_type: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_type: Option<Comments<'a>>,
   },
@@ -1161,11 +1140,9 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_group: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_group: Option<Comments<'a>>,
   },
@@ -1179,11 +1156,9 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_group: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_group: Option<Comments<'a>>,
   },
@@ -1199,7 +1174,6 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments: Option<Comments<'a>>,
   },
@@ -1213,15 +1187,12 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_group: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_group: Option<Comments<'a>>,
   },
@@ -1237,7 +1208,6 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments: Option<Comments<'a>>,
   },
@@ -1246,7 +1216,7 @@ pub enum Type2<'a> {
   /// is the type of the tagged value
   TaggedData {
     /// Tag
-    tag: Option<usize>,
+    tag: Option<u64>,
     /// Type
     t: Type<'a>,
     /// Span
@@ -1254,11 +1224,9 @@ pub enum Type2<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_type: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_type: Option<Comments<'a>>,
   },
@@ -1267,8 +1235,8 @@ pub enum Type2<'a> {
   DataMajorType {
     /// Major type
     mt: u8,
-    /// Constraint
-    constraint: Option<usize>,
+    /// Constraint - Using u64 to support larger values in wasm32
+    constraint: Option<u64>,
     /// Span
     #[cfg(feature = "ast-span")]
     span: Span,
@@ -1283,7 +1251,7 @@ pub enum Type2<'a> {
 }
 
 #[allow(clippy::cognitive_complexity)]
-impl<'a> fmt::Display for Type2<'a> {
+impl fmt::Display for Type2<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     match self {
       Type2::IntValue { value, .. } => write!(f, "{}", value),
@@ -1659,7 +1627,7 @@ impl<'a> From<Type1<'a>> for Type2<'a> {
   }
 }
 
-impl<'a> From<usize> for Type2<'a> {
+impl From<usize> for Type2<'_> {
   fn from(value: usize) -> Self {
     Type2::UintValue {
       value,
@@ -1669,7 +1637,7 @@ impl<'a> From<usize> for Type2<'a> {
   }
 }
 
-impl<'a> From<isize> for Type2<'a> {
+impl From<isize> for Type2<'_> {
   fn from(value: isize) -> Self {
     Type2::IntValue {
       value,
@@ -1679,7 +1647,7 @@ impl<'a> From<isize> for Type2<'a> {
   }
 }
 
-impl<'a> From<f64> for Type2<'a> {
+impl From<f64> for Type2<'_> {
   fn from(value: f64) -> Self {
     Type2::FloatValue {
       value,
@@ -1689,7 +1657,7 @@ impl<'a> From<f64> for Type2<'a> {
   }
 }
 
-impl<'a> From<String> for Type2<'a> {
+impl From<String> for Type2<'_> {
   fn from(value: String) -> Self {
     Type2::TextValue {
       value: value.into(),
@@ -1934,7 +1902,7 @@ impl<'a> From<GroupEntry<'a>> for Group<'a> {
   }
 }
 
-impl<'a> fmt::Display for Group<'a> {
+impl fmt::Display for Group<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut group_str = String::new();
 
@@ -2031,7 +1999,6 @@ pub struct GroupChoice<'a> {
   // No trailing comments since these will be captured by the S ["," S] matching
   // rule
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments_before_grpchoice: Option<Comments<'a>>,
 }
@@ -2089,7 +2056,7 @@ impl<'a> GroupChoice<'a> {
   }
 }
 
-impl<'a> fmt::Display for GroupChoice<'a> {
+impl fmt::Display for GroupChoice<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut gc_str = String::new();
 
@@ -2273,11 +2240,9 @@ pub enum GroupEntry<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     leading_comments: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     trailing_comments: Option<Comments<'a>>,
   },
@@ -2292,11 +2257,9 @@ pub enum GroupEntry<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     leading_comments: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     trailing_comments: Option<Comments<'a>>,
   },
@@ -2312,17 +2275,15 @@ pub enum GroupEntry<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_group: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_group: Option<Comments<'a>>,
   },
 }
 
-impl<'a> GroupEntry<'a> {
+impl GroupEntry<'_> {
   #[cfg(feature = "ast-comments")]
   fn has_trailing_comments(&self) -> bool {
     matches!(self,
@@ -2346,7 +2307,6 @@ pub struct OptionalComma<'a> {
   pub optional_comma: bool,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub trailing_comments: Option<Comments<'a>>,
 
@@ -2354,7 +2314,7 @@ pub struct OptionalComma<'a> {
   pub _a: PhantomData<&'a ()>,
 }
 
-impl<'a> fmt::Display for OptionalComma<'a> {
+impl fmt::Display for OptionalComma<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut optcomma_str = String::new();
 
@@ -2389,7 +2349,7 @@ impl<'a> fmt::Display for OptionalComma<'a> {
   }
 }
 
-impl<'a> OptionalComma<'a> {
+impl OptionalComma<'_> {
   #[cfg(feature = "ast-comments")]
   fn has_trailing_comments(&self) -> bool {
     if let Some(comments) = &self.trailing_comments {
@@ -2400,7 +2360,7 @@ impl<'a> OptionalComma<'a> {
   }
 }
 
-impl<'a> fmt::Display for GroupEntry<'a> {
+impl fmt::Display for GroupEntry<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     match self {
       GroupEntry::ValueMemberKey {
@@ -2540,7 +2500,6 @@ pub struct Occurrence<'a> {
   pub occur: Occur,
 
   #[cfg(feature = "ast-comments")]
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   pub comments: Option<Comments<'a>>,
 
@@ -2548,7 +2507,7 @@ pub struct Occurrence<'a> {
   pub _a: PhantomData<&'a ()>,
 }
 
-impl<'a> fmt::Display for Occurrence<'a> {
+impl fmt::Display for Occurrence<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     #[cfg(feature = "ast-comments")]
     let mut occur_str = self.occur.to_string();
@@ -2582,7 +2541,7 @@ pub struct ValueMemberKeyEntry<'a> {
   pub entry_type: Type<'a>,
 }
 
-impl<'a> fmt::Display for ValueMemberKeyEntry<'a> {
+impl fmt::Display for ValueMemberKeyEntry<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut vmke_str = String::new();
 
@@ -2613,7 +2572,7 @@ pub struct TypeGroupnameEntry<'a> {
   pub generic_args: Option<GenericArgs<'a>>,
 }
 
-impl<'a> fmt::Display for TypeGroupnameEntry<'a> {
+impl fmt::Display for TypeGroupnameEntry<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     let mut tge_str = String::new();
 
@@ -2652,15 +2611,12 @@ pub enum MemberKey<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_before_cut: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_cut: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_arrowmap: Option<Comments<'a>>,
   },
@@ -2675,11 +2631,9 @@ pub enum MemberKey<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_colon: Option<Comments<'a>>,
   },
@@ -2694,16 +2648,13 @@ pub enum MemberKey<'a> {
     span: Span,
 
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments: Option<Comments<'a>>,
     #[cfg(feature = "ast-comments")]
-    #[cfg_attr(target_arch = "wasm32", serde(skip))]
     #[doc(hidden)]
     comments_after_colon: Option<Comments<'a>>,
   },
 
-  #[cfg_attr(target_arch = "wasm32", serde(skip))]
   #[doc(hidden)]
   NonMemberKey {
     non_member_key: NonMemberKey<'a>,
@@ -2714,6 +2665,7 @@ pub enum MemberKey<'a> {
   },
 }
 
+#[cfg_attr(target_arch = "wasm32", derive(Serialize))]
 #[derive(Debug, Clone, PartialEq)]
 #[doc(hidden)]
 pub enum NonMemberKey<'a> {
@@ -2721,7 +2673,7 @@ pub enum NonMemberKey<'a> {
   Type(Type<'a>),
 }
 
-impl<'a> fmt::Display for MemberKey<'a> {
+impl fmt::Display for MemberKey<'_> {
   fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
     match self {
       MemberKey::Type1 {
