@@ -337,7 +337,7 @@ impl<'a> Lexer<'a> {
     self
       .input
       .next()
-      .map(|c| {
+      .inspect(|c| {
         if c.1 == '\n' {
           self.position.line += 1;
           self.position.column = 1;
@@ -348,8 +348,6 @@ impl<'a> Lexer<'a> {
         if !c.1.is_ascii_whitespace() {
           self.position.index = c.0;
         }
-
-        c
       })
       .ok_or_else(|| (self.str_input, self.position, UnableToAdvanceToken).into())
   }
@@ -502,7 +500,7 @@ impl<'a> Lexer<'a> {
                 {
                   Ok((
                     self.position,
-                    Token::TAG(Some(t as u8), Some(self.read_number(idx)?.1 as usize)),
+                    Token::TAG(Some(t as u8), Some(self.read_number(idx)?.1 as u64)),
                   ))
                 }
               }
@@ -958,7 +956,7 @@ impl<'a> Lexer<'a> {
 
     #[cfg(not(target_arch = "wasm32"))]
     {
-      Ok(Token::VALUE(Value::UINT(i)))
+      Ok(Token::VALUE(Value::UINT(i as usize)))
     }
 
     #[cfg(target_arch = "wasm32")]
@@ -968,7 +966,7 @@ impl<'a> Lexer<'a> {
   }
 
   #[cfg(not(target_arch = "wasm32"))]
-  fn read_number(&mut self, idx: usize) -> Result<(usize, usize)> {
+  fn read_number(&mut self, idx: usize) -> Result<(usize, u64)> {
     let mut end_index = idx;
 
     while let Some(&c) = self.peek_char() {
