@@ -417,7 +417,13 @@ impl<'a> JSONValidator<'a> {
                 }
                 ArrayItemToken::Group(group) => jv.visit_group(group)?,
                 ArrayItemToken::Identifier(ident) => jv.visit_identifier(ident)?,
-                _ => (),
+                ArrayItemToken::TaggedData(tag_data) => {
+                  // Handle tagged data
+                  jv.add_error(format!(
+                    "Tagged data not supported in arrays: {:?}",
+                    tag_data
+                  ))
+                }
               }
 
               if self.is_multi_type_choice && jv.errors.is_empty() {
@@ -461,9 +467,22 @@ impl<'a> JSONValidator<'a> {
                 ArrayItemToken::Range(lower, upper, is_inclusive) => {
                   jv.visit_range(lower, upper, *is_inclusive)?
                 }
-                ArrayItemToken::Group(group) => jv.visit_group(group)?,
+
                 ArrayItemToken::Identifier(ident) => jv.visit_identifier(ident)?,
-                _ => (),
+                ArrayItemToken::TaggedData(tag_data) => {
+                  // Handle tagged data
+                  jv.add_error(format!(
+                    "Tagged data not supported in arrays: {:?}",
+                    tag_data
+                  ))
+                }
+                // Special nested array handling - using Group variant instead
+                // as Array variant doesn't exist
+                ArrayItemToken::Group(group) if v.is_array() => {
+                  // Special handling for nested arrays
+                  jv.visit_group(group)?;
+                }
+                ArrayItemToken::Group(group) => jv.visit_group(group)?,
               }
 
               self.errors.append(&mut jv.errors);
@@ -554,7 +573,7 @@ impl<'a> JSONValidator<'a> {
   }
 }
 
-impl<'a, 'b> Validator<'a, 'b, Error> for JSONValidator<'a> {
+impl<'a> Validator<'a, '_, Error> for JSONValidator<'a> {
   /// Validate
   fn validate(&mut self) -> std::result::Result<(), Error> {
     for r in self.cddl.rules.iter() {
@@ -589,7 +608,7 @@ impl<'a, 'b> Validator<'a, 'b, Error> for JSONValidator<'a> {
   }
 }
 
-impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
+impl<'a> Visitor<'a, '_, Error> for JSONValidator<'a> {
   fn visit_type_rule(&mut self, tr: &TypeRule<'a>) -> visitor::Result<Error> {
     if let Some(gp) = &tr.generic_params {
       if let Some(gr) = self
@@ -675,6 +694,42 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
   }
 
   fn visit_type(&mut self, t: &Type<'a>) -> visitor::Result<Error> {
+    // Special case for nested array in literal position
+    if let Value::Array(outer_array) = &self.json {
+      if let Some(idx) = self.group_entry_idx {
+        // We're processing a specific array item
+        if let Some(item) = outer_array.get(idx) {
+          if item.is_array() {
+            // This is a nested array, check if we're supposed to validate against an array type
+            for tc in t.type_choices.iter() {
+              if let Type2::Array { .. } = &tc.type1.type2 {
+                #[cfg(all(feature = "additional-controls", target_arch = "wasm32"))]
+                let mut jv =
+                  JSONValidator::new(self.cddl, item.clone(), self.enabled_features.clone());
+                #[cfg(all(feature = "additional-controls", not(target_arch = "wasm32")))]
+                let mut jv = JSONValidator::new(self.cddl, item.clone(), self.enabled_features);
+                #[cfg(not(feature = "additional-controls"))]
+                let mut jv = JSONValidator::new(self.cddl, item.clone());
+
+                jv.generic_rules = self.generic_rules.clone();
+                jv.eval_generic_rule = self.eval_generic_rule;
+                jv.is_multi_type_choice = self.is_multi_type_choice;
+
+                let _ = write!(jv.json_location, "{}/{}", self.json_location, idx);
+
+                // Visit the type choice with the inner array value
+                jv.visit_type_choice(tc)?;
+
+                self.errors.append(&mut jv.errors);
+                return Ok(());
+              }
+            }
+          }
+        }
+      }
+    }
+
+    // Regular type processing
     if t.type_choices.len() > 1 {
       self.is_multi_type_choice = true;
     }
@@ -878,222 +933,154 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
       return self.validate_array_items(&ArrayItemToken::Range(lower, upper, is_inclusive));
     }
 
-    match lower {
-      Type2::IntValue { value: l, .. } => match upper {
-        Type2::IntValue { value: u, .. } => {
-          let error_str = if is_inclusive {
-            format!(
-              "expected integer to be in range {} <= value <= {}, got {}",
-              l, u, self.json
-            )
-          } else {
-            format!(
-              "expected integer to be in range {} < value < {}, got {}",
-              l, u, self.json
-            )
-          };
+    match (lower, upper) {
+      (Type2::IntValue { value: l, .. }, Type2::IntValue { value: u, .. }) => {
+        let error_str = if is_inclusive {
+          format!(
+            "expected integer to be in range {} <= value <= {}, got {}",
+            l, u, self.json
+          )
+        } else {
+          // Changed: For ... ranges, make it inclusive on lower bound, exclusive on upper bound
+          format!(
+            "expected integer to be in range {} <= value < {}, got {}",
+            l, u, self.json
+          )
+        };
 
-          match &self.json {
-            Value::Number(n) => {
-              if let Some(i) = n.as_i64() {
-                if is_inclusive {
-                  if i < *l as i64 || i > *u as i64 {
-                    self.add_error(error_str);
-                  } else {
-                    return Ok(());
-                  }
-                } else if i <= *l as i64 || i >= *u as i64 {
+        match &self.json {
+          Value::Number(n) => {
+            if let Some(i) = n.as_i64() {
+              if is_inclusive {
+                if i < *l as i64 || i > *u as i64 {
                   self.add_error(error_str);
-                  return Ok(());
-                } else {
-                  return Ok(());
                 }
               } else {
-                self.add_error(error_str);
-                return Ok(());
+                // Changed: For ... ranges, make it inclusive on lower bound, exclusive on upper bound
+                if i < *l as i64 || i >= *u as i64 {
+                  self.add_error(error_str);
+                }
               }
-            }
-            _ => {
+            } else {
               self.add_error(error_str);
               return Ok(());
             }
           }
+          _ => {
+            self.add_error(format!(
+              "invalid cddl range. value must be an integer type. got {}",
+              self.json
+            ));
+            return Ok(());
+          }
         }
-        Type2::UintValue { value: u, .. } => {
-          let error_str = if is_inclusive {
-            format!(
-              "expected integer to be in range {} <= value <= {}, got {}",
-              l, u, self.json
-            )
-          } else {
-            format!(
-              "expected integer to be in range {} < value < {}, got {}",
-              l, u, self.json
-            )
-          };
+      }
+      (Type2::UintValue { value: l, .. }, Type2::UintValue { value: u, .. }) => {
+        let error_str = if is_inclusive {
+          format!(
+            "expected uint to be in range {} <= value <= {}, got {}",
+            l, u, self.json
+          )
+        } else {
+          // Changed: For ... ranges, make it inclusive on lower bound, exclusive on upper bound
+          format!(
+            "expected uint to be in range {} <= value < {}, got {}",
+            l, u, self.json
+          )
+        };
 
-          match &self.json {
-            Value::Number(n) => {
-              if let Some(i) = n.as_i64() {
-                if is_inclusive {
-                  if i < *l as i64 || i > *u as i64 {
-                    self.add_error(error_str);
-                  } else {
-                    return Ok(());
-                  }
-                } else if i <= *l as i64 || i >= *u as i64 {
+        match &self.json {
+          Value::Number(n) => {
+            if let Some(i) = n.as_u64() {
+              // Fix: Cast all usize values to u64
+              let lower = *l as u64;
+              let upper = *u as u64;
+              if is_inclusive {
+                if i < lower || i > upper {
                   self.add_error(error_str);
-                  return Ok(());
-                } else {
-                  return Ok(());
                 }
-              } else {
+              } else if i < lower || i >= upper {
                 self.add_error(error_str);
-                return Ok(());
               }
-            }
-            _ => {
+            } else {
               self.add_error(error_str);
               return Ok(());
             }
           }
-        }
-        _ => {
-          self.add_error(format!(
-            "invalid cddl range. upper value must be an integer type. got {}",
-            upper
-          ));
-          return Ok(());
-        }
-      },
-      Type2::UintValue { value: l, .. } => match upper {
-        Type2::UintValue { value: u, .. } => {
-          let error_str = if is_inclusive {
-            format!(
-              "expected uint to be in range {} <= value <= {}, got {}",
-              l, u, self.json
-            )
-          } else {
-            format!(
-              "expected uint to be in range {} < value < {}, got {}",
-              l, u, self.json
-            )
-          };
-
-          match &self.json {
-            Value::Number(n) => {
-              if let Some(i) = n.as_u64() {
-                if is_inclusive {
-                  if i < *l as u64 || i > *u as u64 {
-                    self.add_error(error_str);
-                  } else {
-                    return Ok(());
-                  }
-                } else if i <= *l as u64 || i >= *u as u64 {
+          Value::String(s) => match self.ctrl {
+            Some(ControlOperator::SIZE) => {
+              let len = s.len() as u64; // Fix: Convert len to u64
+                                        // Fix: Cast all usize values to u64
+              let lower = *l as u64;
+              let upper = *u as u64;
+              if is_inclusive {
+                if len < lower || len > upper {
                   self.add_error(error_str);
-                  return Ok(());
-                } else {
-                  return Ok(());
                 }
-              } else {
+              } else if len < lower || len >= upper {
                 self.add_error(error_str);
-                return Ok(());
               }
             }
-            Value::String(s) => match self.ctrl {
-              Some(ControlOperator::SIZE) => {
-                let len = s.len();
-                let s = s.clone();
-                if is_inclusive {
-                  if s.len() < *l || s.len() > *u {
-                    self.add_error(format!(
-                      "expected \"{}\" string length to be in the range {} <= value <= {}, got {}",
-                      s, l, u, len
-                    ));
-                  }
+            _ => {
+              self.add_error("string value cannot be validated against a range without the .size control operator".to_string());
+              return Ok(());
+            }
+          },
+          _ => {
+            self.add_error(format!(
+              "invalid cddl range. value must be a uint type. got {}",
+              self.json
+            ));
+            return Ok(());
+          }
+        }
+      }
+      (Type2::FloatValue { value: l, .. }, Type2::FloatValue { value: u, .. }) => {
+        let error_str = if is_inclusive {
+          format!(
+            "expected float to be in range {} <= value <= {}, got {}",
+            l, u, self.json
+          )
+        } else {
+          // Changed: For ... ranges, make it inclusive on lower bound, exclusive on upper bound
+          format!(
+            "expected float to be in range {} <= value < {}, got {}",
+            l, u, self.json
+          )
+        };
 
-                  return Ok(());
-                } else if s.len() <= *l || s.len() >= *u {
-                  self.add_error(format!(
-                    "expected \"{}\" string length to be in the range {} < value < {}, got {}",
-                    s, l, u, len
-                  ));
-                  return Ok(());
+        match &self.json {
+          Value::Number(n) => {
+            if let Some(f) = n.as_f64() {
+              if is_inclusive {
+                if f < *l || f > *u {
+                  self.add_error(error_str);
+                }
+              } else {
+                // Changed: For ... ranges, make it inclusive on lower bound, exclusive on upper bound
+                if f < *l || f >= *u {
+                  self.add_error(error_str);
                 }
               }
-              _ => {
-                self.add_error("string value cannot be validated against a range without the .size control operator".to_string());
-                return Ok(());
-              }
-            },
-            _ => {
+            } else {
               self.add_error(error_str);
               return Ok(());
             }
           }
-        }
-        _ => {
-          self.add_error(format!(
-            "invalid cddl range. upper value must be a uint type. got {}",
-            upper
-          ));
-          return Ok(());
-        }
-      },
-      Type2::FloatValue { value: l, .. } => match upper {
-        Type2::FloatValue { value: u, .. } => {
-          let error_str = if is_inclusive {
-            format!(
-              "expected float to be in range {} <= value <= {}, got {}",
-              l, u, self.json
-            )
-          } else {
-            format!(
-              "expected float to be in range {} < value < {}, got {}",
-              l, u, self.json
-            )
-          };
-
-          match &self.json {
-            Value::Number(n) => {
-              if let Some(f) = n.as_f64() {
-                if is_inclusive {
-                  if f < *l || f > *u {
-                    self.add_error(error_str);
-                  } else {
-                    return Ok(());
-                  }
-                } else if f <= *l || f >= *u {
-                  self.add_error(error_str);
-                  return Ok(());
-                } else {
-                  return Ok(());
-                }
-              } else {
-                self.add_error(error_str);
-                return Ok(());
-              }
-            }
-            _ => {
-              self.add_error(error_str);
-              return Ok(());
-            }
+          _ => {
+            self.add_error(format!(
+              "invalid cddl range. value must be a float type. got {}",
+              self.json
+            ));
+            return Ok(());
           }
         }
-        _ => {
-          self.add_error(format!(
-            "invalid cddl range. upper value must be a float type. got {}",
-            upper
-          ));
-          return Ok(());
-        }
-      },
+      }
       _ => {
         self.add_error(
           "invalid cddl range. upper and lower values must be either integers or floats"
             .to_string(),
         );
-
         return Ok(());
       }
     }
@@ -1118,12 +1105,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
       } = controller
       {
         if let Some(name) = self.eval_generic_rule {
-          if let Some(gr) = self
-            .generic_rules
-            .iter()
-            .cloned()
-            .find(|gr| gr.name == name)
-          {
+          if let Some(gr) = self.generic_rules.iter().find(|r| r.name == name).cloned() {
             for (idx, gp) in gr.params.iter().enumerate() {
               if let Some(arg) = gr.args.get(idx) {
                 if *gp == target_ident.ident {
@@ -1142,12 +1124,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
       }
 
       if let Some(name) = self.eval_generic_rule {
-        if let Some(gr) = self
-          .generic_rules
-          .iter()
-          .cloned()
-          .find(|gr| gr.name == name)
-        {
+        if let Some(gr) = self.generic_rules.iter().find(|r| r.name == name).cloned() {
           for (idx, gp) in gr.params.iter().enumerate() {
             if let Some(arg) = gr.args.get(idx) {
               if *gp == target_ident.ident {
@@ -1287,15 +1264,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
         let error_count = self.errors.len();
         self.visit_type2(target)?;
         if self.errors.len() != error_count {
-          #[cfg(feature = "ast-span")]
-          if let Some(Occur::Optional { .. }) = self.occurrence.take() {
-            self.add_error(format!(
-              "expected default value {}, got {}",
-              controller, self.json
-            ));
-          }
-          #[cfg(not(feature = "ast-span"))]
-          if let Some(Occur::Optional {}) = self.occurrence.take() {
+          if let Some(Occur::Optional { .. }) = self.occurrence.as_ref() {
             self.add_error(format!(
               "expected default value {}, got {}",
               controller, self.json
@@ -1604,6 +1573,11 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
 
           Ok(())
         }
+        // Special case for nested array validation
+        _ if matches!(self.json, Value::Array(_)) => {
+          self.validate_array_items(&ArrayItemToken::Group(group))?;
+          Ok(())
+        }
         _ => {
           self.add_error(format!("expected array type, got {}", self.json));
           Ok(())
@@ -1619,7 +1593,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
             if let Some(gr) = self
               .generic_rules
               .iter_mut()
-              .find(|gr| gr.name == ident.ident)
+              .find(|r| r.name == ident.ident)
             {
               for arg in ga.args.iter() {
                 gr.args.push((*arg.arg).clone());
@@ -1682,7 +1656,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
             if let Some(gr) = self
               .generic_rules
               .iter_mut()
-              .find(|gr| gr.name == ident.ident)
+              .find(|r| r.name == ident.ident)
             {
               for arg in ga.args.iter() {
                 gr.args.push((*arg.arg).clone());
@@ -1817,8 +1791,8 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
       if let Some(gr) = self
         .generic_rules
         .iter()
+        .find(|&gr| gr.name == name)
         .cloned()
-        .find(|gr| gr.name == name)
       {
         for (idx, gp) in gr.params.iter().enumerate() {
           if *gp == ident.ident {
@@ -1842,6 +1816,20 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
       return Ok(());
     }
 
+    // Special case for array values - check if we're in an array context and this
+    // is a reference to another array type
+    if let Value::Array(_) = &self.json {
+      if let Some(rule) = rule_from_ident(self.cddl, ident) {
+        if let Rule::Type { rule, .. } = rule {
+          for tc in rule.value.type_choices.iter() {
+            if let Type2::Array { .. } = &tc.type1.type2 {
+              return self.visit_type_choice(tc);
+            }
+          }
+        }
+      }
+    }
+
     match &self.json {
       Value::Null if is_ident_null_data_type(self.cddl, ident) => Ok(()),
       Value::Bool(b) => {
@@ -1917,7 +1905,17 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
 
         Ok(())
       }
-      Value::Array(_) => self.validate_array_items(&ArrayItemToken::Identifier(ident)),
+      Value::Array(_) => {
+        // For arrays, check if this identifier refers to an array type rule first
+        if let Some(r) = rule_from_ident(self.cddl, ident) {
+          if is_array_type_rule(self.cddl, ident) {
+            return self.visit_rule(r);
+          }
+        }
+
+        // Then fallback to normal array validation
+        self.validate_array_items(&ArrayItemToken::Identifier(ident))
+      }
       Value::Object(o) => match &self.occurrence {
         #[cfg(feature = "ast-span")]
         Some(Occur::Optional { .. }) | None => {
@@ -2395,7 +2393,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
         Value::Number(n) => match n.as_f64() {
           Some(f) => match &self.ctrl {
             Some(ControlOperator::NE) | Some(ControlOperator::DEFAULT)
-              if (f - *v).abs() > std::f64::EPSILON =>
+              if (f - *v).abs() > f64::EPSILON =>
             {
               None
             }
@@ -2405,7 +2403,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
             Some(ControlOperator::GE) if f >= *v => None,
             #[cfg(feature = "additional-controls")]
             Some(ControlOperator::PLUS) => {
-              if (f - *v).abs() < std::f64::EPSILON {
+              if (f - *v).abs() < f64::EPSILON {
                 None
               } else {
                 Some(format!("expected computed .plus value {}, got {}", v, n))
@@ -2413,7 +2411,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
             }
             #[cfg(feature = "additional-controls")]
             None | Some(ControlOperator::FEATURE) => {
-              if (f - *v).abs() < std::f64::EPSILON {
+              if (f - *v).abs() < f64::EPSILON {
                 None
               } else {
                 Some(format!("expected value {}, got {}", v, n))
@@ -2421,7 +2419,7 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
             }
             #[cfg(not(feature = "additional-controls"))]
             None => {
-              if (f - *v).abs() < std::f64::EPSILON {
+              if (f - *v).abs() < f64::EPSILON {
                 None
               } else {
                 Some(format!("expected value {}, got {}", v, n))
@@ -2527,6 +2525,23 @@ impl<'a, 'b> Visitor<'a, 'b, Error> for JSONValidator<'a> {
   }
 }
 
+// Helper function to check if an identifier refers to an array type rule
+fn is_array_type_rule<'a>(cddl: &'a CDDL<'a>, ident: &Identifier<'a>) -> bool {
+  for r in cddl.rules.iter() {
+    if let Rule::Type { rule, .. } = r {
+      if rule.name.ident == ident.ident {
+        // Check if the type rule defines an array
+        for tc in rule.value.type_choices.iter() {
+          if let Type2::Array { .. } = &tc.type1.type2 {
+            return true;
+          }
+        }
+      }
+    }
+  }
+  false
+}
+
 #[cfg(test)]
 #[cfg(not(target_arch = "wasm32"))]
 mod tests {
@@ -2801,4 +2816,185 @@ mod tests {
 
     Ok(())
   }
+
+  #[test]
+  fn validate_range_operators() -> std::result::Result<(), Box<dyn std::error::Error>> {
+    // Test inclusive range (..)
+    let cddl = indoc!(
+      r#"
+        thing = 5..10
+        "#
+    );
+
+    let json = serde_json::json!(5); // Should pass
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(10); // Should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(4); // Should fail
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    let json = serde_json::json!(11); // Should fail
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    // Test inclusive-exclusive range (...)
+    let cddl = indoc!(
+      r#"
+        thing = 5...10
+        "#
+    );
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+
+    let json = serde_json::json!(5); // Should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(9); // Should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(10); // Should fail (exclusive upper bound)
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    let json = serde_json::json!(4); // Should fail
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    // Test inclusive-exclusive range with floats
+    let cddl = indoc!(
+      r#"
+        thing = 1.5...3.5
+        "#
+    );
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+
+    let json = serde_json::json!(1.5); // Should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(2.5); // Should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!(3.5); // Should fail (exclusive upper bound)
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    let json = serde_json::json!(1.0); // Should fail
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    // Test range with string length using .size control
+    let cddl = indoc!(
+      r#"
+        thing = tstr .size (2...5)
+        "#
+    );
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+
+    let json = serde_json::json!("ab"); // Length 2 - should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!("abcd"); // Length 4 - should pass
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_ok());
+
+    let json = serde_json::json!("abcde"); // Length 5 - should fail (exclusive)
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    let json = serde_json::json!("a"); // Length 1 - should fail
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    assert!(jv.validate().is_err());
+
+    Ok(())
+  }
+
+  #[test]
+  fn validate_nested_arrays() -> std::result::Result<(), Box<dyn std::error::Error>> {
+    // Test with explicit array type
+    let cddl = indoc!(
+      r#"
+        array = [0, [* int]]
+      "#
+    );
+
+    let json = r#"[0, [1, 2]]"#;
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+    let json = serde_json::from_str::<serde_json::Value>(json).map_err(json::Error::JSONParsing)?;
+
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    jv.validate()?;
+
+    // Test with named arrays
+    let cddl = indoc!(
+      r#"
+        root = [0, inner]
+        inner = [* int]
+      "#
+    );
+
+    let json = r#"[0, [1, 2]]"#;
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+    let json = serde_json::from_str::<serde_json::Value>(json).map_err(json::Error::JSONParsing)?;
+
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    jv.validate()?;
+
+    // Test with explicit array literals
+    let cddl = indoc!(
+      r#"
+        direct = [1, [2, 3]]
+      "#
+    );
+
+    let json = r#"[1, [2, 3]]"#;
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+    let json = serde_json::from_str::<serde_json::Value>(json).map_err(json::Error::JSONParsing)?;
+
+    let mut jv = JSONValidator::new(&cddl, json, None);
+    jv.validate()?; // If this passes, our fix works
+
+    Ok(())
+  }
+
+  #[test]
+  fn validate_direct_nested_array() -> std::result::Result<(), Box<dyn std::error::Error>> {
+    // Simplest possible case with explicit literal array
+    let cddl = indoc!(
+      r#"
+        direct = [1, [2, 3]]
+        "#
+    );
+
+    let json = r#"[1, [2, 3]]"#;
+    let cddl = cddl_from_str(cddl, true).map_err(json::Error::CDDLParsing)?;
+    let json = serde_json::from_str::<serde_json::Value>(json).map_err(json::Error::JSONParsing)?;
+
+    let mut jv = JSONValidator::new(&cddl, json, None);
+
+    // Print detailed information for debugging
+    match jv.validate() {
+      Ok(_) => println!("Validation successful!"),
+      Err(e) => {
+        eprintln!("Validation error: {}", e);
+        if let Error::Validation(errors) = &e {
+          for (i, err) in errors.iter().enumerate() {
+            eprintln!("Error {}: {} at {}", i, err.reason, err.json_location);
+          }
+        }
+        return Err(e.into());
+      }
+    }
+
+    Ok(())
+  }
 }
